#!/usr/bin/perl
# index.cgi

#$no_acl_check = 1;

require './batch-apache-lib.pl';

my (@new_domains, @bad_domains);
my %bad_reason;
my $address;
my $directives;

&ReadParse;
&process;
&output;

sub process {
    # modifies globals: @bad_domains, %bad_reason, @new_domains, $address.

    foreign_require("apache", "apache-lib.pl");

    my $error;
    my $rawlist = $in{'domains'};
    my @raw_domains = split '\n', $rawlist;
    my $home_path = $config{'home_path'} || { $error = 1 };
    my $user = $config{'user'} || { $error = 1 };
    if ($config{'should_default_webmaster'}) {
	$config{'default_webmaster'} || { $error = 1 };
    }
    my $should_copyindex = $config{'should_copyindex'} || 0;
    my $indexfile;
    if ($should_copyindex) {
	$indexfile = $config{'indexfile'} || { $error = 1 };
    }

    if ($error) { 
	&error("Missing Configuration information. Please select \"Module Config\"."); 
    }
    #getting htconf
    foreign_require("apache", "apache-lib.pl");

    %apache_config = foreign_config('apache');
    if ($apache_config{'httpd_conf'}) { 
	$htconf = $apache_config{'httpd_conf'}; 
    } else {
	$htconf = "$apache_config{'httpd_dir'}/conf/httpd.conf";
    }
    
    #getting names of all servers:
    my ($conf, @virt, %vserv, $ns, $v, $vm);

    $conf = foreign_call('apache', 'get_config');
    @virt = foreign_call('apache', find_directive_struct, "VirtualHost", $conf);


    $ns = foreign_call('apache', 'find_directive', "ServerName", $conf);
    $ns && ($vserv{$ns} = $ns);              

    
    foreach $v (@virt) {
	$vm = $v->{'members'};
	$ns = foreign_call('apache', 'find_vdirective', "ServerName", $vm, $conf);
	$ns && ($vserv{$ns} = $ns);
    }
    

    # validate IP address
    my $default_address = to_ipaddress((&get_system_hostname));
    $address = $config{'addr'} || $default_address;

    if ($address !~ /\S/) { &error("IP Address must not contain spaces.") };
    if (!&check_ipaddress($address)) { 
	&error("Address $address is not a valid IP"); 
    }
    
    unless (-d $home_path) {
	&error("Path $home_path does not exist.");
    }

    # name must have one or more '.', no illegal characters, no repeats.
    foreach $domain (@raw_domains) {
	$domain =~ s/\s$//;
	if ($domain =~ /^\s*$/) { # entirely whitespace
	    next;
	} elsif ($domain !~ /^\S+$/) {
	    push @bad_domains, $domain;
	    $bad_reason{$domain} = "Bad format: Domain contains space or other bad character.";
	} elsif ($domain !~ /\./) {
	    push @bad_domains, $domain;
	    $bad_reason{$domain} = "Bad format: Domain does not contain a dot.";
	} elsif ($vserv{$domain}) {
	    push @bad_domains, $domain;
            $bad_reason{$domain} = "Domain already exists on this server.";
        } else {
	    push @new_domains, $domain;
	}
    }

    # build virtualhost directives and create directories

    my $fullpath;
    my $webmaster = $config{'default_webmaster'};
    foreach $domain (@new_domains) {
	$fullpath = "$home_path/$domain";
	unless ($config{'should_default_webmaster'}) {
	    $webmaster = "webmaster\@$domain";
	}
	$directives .= <<EOF;

<VirtualHost $address>
ServerAdmin $webmaster 
ServerName $domain   
ServerAlias $domain *.$domain   
DocumentRoot $fullpath/htdocs
</VirtualHost>
Alias /$domain $fullpath/htdocs
EOF
	
        # create directories
        system("mkdir $fullpath");
	system("mkdir $fullpath/htdocs");
	system("mkdir $fullpath/cgi-bin");
	if ($should_copyindex) {
	    system("ln -s $indexfile $fullpath/htdocs/index.html");
	}

	system("chown $user:$user $fullpath -R");
	system("chmod 775 $fullpath -R");
	
    }

    open(HTCONF, ">>$htconf") 
	or &error("Couldn't open $htconf for writing. $!");
    print HTCONF $directives;
    close(HTCONF);

}

sub output {

    my $domain;

    &header("Adding Apache Domains", "", "");
    if (@bad_domains) {
	print "Errors with the following ", int(@bad_domains), " domain(s):<P>\n";
	print "<table>\n";
	print "<tr><td><strong>Domain Name &nbsp;&nbsp;&nbsp;</strong></td>";
	print "<td><strong> Reason</strong></td></tr>";
	foreach $domain (@bad_domains) {
	    print "<tr><td>$domain</td><td>$bad_reason{$domain}</td></tr>\n";
	}
	print "</table>\n";
	print "<hr>\n";
    }
    if (@new_domains) {
	print "Added the following ", int(@new_domains), " domain(s):<P>\n";
	print "<table>\n";
	print "<tr><td><strong>Domain Name &nbsp;&nbsp;&nbsp;</strong></td></tr>";
	foreach $domain (@new_domains) {
	    print "<tr><td>$domain</td></tr>\n";
	}
	print "</table>\n";
    
	print "<hr>\n";
    } else {
	print "Did not add any new domains.\n";
	print "<hr>\n";
    }


    print "To restart Apache, please <a href=\"/apache/restart.cgi?redir=/batch-apache/back.html\">link here</a>.<P>\n";

    # chain to another server, if necessary.

    my $out_rawlist = join "\n", @new_domains;

    if ($config{'follow'}) {
        print "<form action=\"$config{'follow'}\" method=\"post\">\n";
        print "<input type=\"hidden\" name=\"secondary\" value=\"0\">\n";
        print "<input type=\"hidden\" name=\"ip\" value=\"$address\">\n";
        print "<input type=\"hidden\" name=\"domains\" value=\"$out_rawlist\">\n";
        print "<input type=\"submit\" value=\"Add Master DNS\"><P>\n";
    }

    &footer("/", "index");
    
}


